"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nitro = void 0;
const h3_1 = require("h3");
const vite_1 = require("vite");
const path = require("path");
const build_server_1 = require("./build-server");
const build_ssr_1 = require("./build-ssr");
const page_endpoints_1 = require("./plugins/page-endpoints");
const get_page_handlers_1 = require("./utils/get-page-handlers");
const build_sitemap_1 = require("./build-sitemap");
const dev_server_plugin_1 = require("./plugins/dev-server-plugin");
const load_esm_1 = require("./utils/load-esm");
let clientOutputPath = '';
function nitro(options, nitroOptions) {
    const workspaceRoot = options?.workspaceRoot ?? process.cwd();
    const isTest = process.env['NODE_ENV'] === 'test' || !!process.env['VITEST'];
    const apiPrefix = `/${nitroOptions?.runtimeConfig?.['apiPrefix'] ?? 'api'}`;
    let isBuild = false;
    let isServe = false;
    let ssrBuild = false;
    let config;
    let nitroConfig;
    return [
        (options?.ssr
            ? (0, dev_server_plugin_1.devServerPlugin)({
                entryServer: options?.entryServer,
                index: options?.index,
            })
            : false),
        {
            name: '@analogjs/vite-plugin-nitro',
            async config(_config, { command }) {
                isServe = command === 'serve';
                isBuild = command === 'build';
                ssrBuild = _config.build?.ssr === true;
                config = _config;
                const rootDir = config.root || '.';
                const buildPreset = process.env['BUILD_PRESET'] ??
                    nitroOptions?.preset;
                const pageHandlers = (0, get_page_handlers_1.getPageHandlers)({ workspaceRoot, rootDir });
                nitroConfig = {
                    rootDir,
                    preset: buildPreset,
                    logLevel: nitroOptions?.logLevel || 0,
                    srcDir: (0, vite_1.normalizePath)(`${rootDir}/src/server`),
                    scanDirs: [(0, vite_1.normalizePath)(`${rootDir}/src/server`)],
                    output: {
                        dir: (0, vite_1.normalizePath)(path.resolve(workspaceRoot, 'dist', rootDir, 'analog')),
                        publicDir: (0, vite_1.normalizePath)(path.resolve(workspaceRoot, 'dist', rootDir, 'analog/public')),
                        ...nitroOptions?.output,
                    },
                    buildDir: (0, vite_1.normalizePath)(path.resolve(workspaceRoot, 'dist', rootDir, '.nitro')),
                    typescript: {
                        generateTsConfig: false,
                    },
                    runtimeConfig: { ...nitroOptions?.runtimeConfig },
                    rollupConfig: {
                        onwarn(warning) {
                            if (warning.message.includes('empty chunk') &&
                                warning.message.endsWith('.server')) {
                                return;
                            }
                        },
                        plugins: [(0, page_endpoints_1.pageEndpointsPlugin)()],
                    },
                    handlers: [
                        {
                            handler: (0, vite_1.normalizePath)(`${__dirname}/runtime/api-middleware`),
                            middleware: true,
                        },
                        ...pageHandlers,
                    ],
                };
                if (isVercelPreset(buildPreset)) {
                    nitroConfig = withVercelOutputAPI(nitroConfig, workspaceRoot);
                }
                if (!ssrBuild && !isTest) {
                    // store the client output path for the SSR build config
                    clientOutputPath = path.resolve(rootDir, config.build?.outDir || 'dist/client');
                }
                nitroConfig.alias = {
                    '#analog/ssr': (0, vite_1.normalizePath)(path.resolve(workspaceRoot, 'dist', rootDir, 'ssr/main.server')),
                    '#analog/index': (0, vite_1.normalizePath)(path.resolve(clientOutputPath, 'index.html')),
                    ...nitroOptions?.alias,
                };
                if (isBuild) {
                    if (isEmptyPrerenderRoutes(options)) {
                        nitroConfig.prerender = {};
                        nitroConfig.prerender.routes = ['/'];
                    }
                    if (options?.prerender) {
                        nitroConfig.prerender = nitroConfig.prerender ?? {};
                        nitroConfig.prerender.crawlLinks = options?.prerender?.discover;
                        const prerenderRoutes = options?.prerender?.routes;
                        if (isArrayWithElements(prerenderRoutes)) {
                            nitroConfig.prerender.routes = prerenderRoutes;
                        }
                        else if (typeof prerenderRoutes === 'function') {
                            nitroConfig.prerender.routes = await prerenderRoutes();
                        }
                    }
                    if (ssrBuild) {
                        nitroConfig = {
                            ...nitroConfig,
                            publicAssets: [{ dir: clientOutputPath }],
                            serverAssets: [
                                {
                                    baseName: 'public',
                                    dir: clientOutputPath,
                                },
                            ],
                            externals: {
                                ...nitroOptions?.externals,
                                external: [
                                    'rxjs',
                                    'node-fetch-native/dist/polyfill',
                                    ...(nitroOptions?.externals?.external || []),
                                ],
                            },
                            moduleSideEffects: [
                                'zone.js/node',
                                'zone.js/fesm2015/zone-node',
                                ...(nitroOptions?.moduleSideEffects || []),
                            ],
                            renderer: (0, vite_1.normalizePath)(`${__dirname}/runtime/renderer`),
                            handlers: [
                                {
                                    handler: (0, vite_1.normalizePath)(`${__dirname}/runtime/api-middleware`),
                                    middleware: true,
                                },
                                ...pageHandlers,
                            ],
                        };
                    }
                }
                nitroConfig = {
                    ...nitroConfig,
                    ...nitroOptions,
                };
            },
            async configureServer(viteServer) {
                if (isServe && !isTest) {
                    const { createNitro, createDevServer, build } = await (0, load_esm_1.loadEsmModule)('nitropack');
                    const nitro = await createNitro({
                        dev: true,
                        ...nitroConfig,
                    });
                    const server = createDevServer(nitro);
                    await build(nitro);
                    viteServer.middlewares.use(apiPrefix, (0, h3_1.toNodeListener)(server.app));
                    viteServer.httpServer?.once('listening', () => {
                        process.env['ANALOG_HOST'] = !viteServer.config.server.host
                            ? 'localhost'
                            : viteServer.config.server.host;
                        process.env['ANALOG_PORT'] = `${viteServer.config.server.port}`;
                    });
                    console.log(`\n\nThe server endpoints are accessible under the "${apiPrefix}" path.`);
                }
            },
            async closeBundle() {
                if (ssrBuild) {
                    return;
                }
                if (isBuild) {
                    if (options?.ssr) {
                        console.log('Building SSR application...');
                        await (0, build_ssr_1.buildSSRApp)(config, options);
                    }
                    await (0, build_server_1.buildServer)(options, nitroConfig);
                    if (options?.prerender?.sitemap) {
                        console.log('Building Sitemap...');
                        // sitemap needs to be built after all directories are built
                        await (0, build_sitemap_1.buildSitemap)(config, options.prerender.sitemap, options.prerender.routes, nitroConfig.output?.publicDir);
                    }
                    console.log(`\n\nThe '@analogjs/platform' server has been successfully built.`);
                }
            },
        },
    ];
}
exports.nitro = nitro;
function isEmptyPrerenderRoutes(options) {
    if (!options || isArrayWithElements(options?.prerender?.routes)) {
        return false;
    }
    return !options.prerender?.routes;
}
function isArrayWithElements(arr) {
    return !!(Array.isArray(arr) && arr.length);
}
const isVercelPreset = (buildPreset) => process.env['VERCEL'] ||
    (buildPreset && buildPreset.toLowerCase().includes('vercel'));
const withVercelOutputAPI = (nitroConfig, workspaceRoot) => ({
    ...nitroConfig,
    output: {
        ...nitroConfig?.output,
        dir: (0, vite_1.normalizePath)(path.resolve(workspaceRoot, '.vercel', 'output')),
        publicDir: (0, vite_1.normalizePath)(path.resolve(workspaceRoot, '.vercel', 'output/static')),
    },
});
//# sourceMappingURL=vite-plugin-nitro.js.map